import React, { useContext, useRef, useState } from 'react';
import { useLocation, matchPath } from 'react-router-dom';
import { GamesContext } from '../../context/games';

export default function Message2Opponent({ dispatchGames }) {
  const games = useContext(GamesContext);
  const { pathname } = useLocation();
  const [value, setValue] = useState('');
  const syncTimeoutRef = useRef(null);

  if (syncTimeoutRef.current === null) { // <<--- Absorb external value if there is no scheduled sync
    if (matchPath('/games/new', pathname)) {
      if (value !== games.newGame.message2opponent)
        setValue(games.newGame.message2opponent);
    } else if (value !== '') {
      setValue('');
    }
  }

  /* --- */

  const sync = (message2opponent) => {
    syncTimeoutRef.current = null;

    if (matchPath('/games/new', pathname))
      return dispatchGames({ type: 'nextNewGame', message2opponent });

    console.warn('unknown path');
  }

  const update = (value) => {
    setValue(value);

    if (syncTimeoutRef.current)
      clearTimeout(syncTimeoutRef.current); // <<--- Cancel previous sync

    syncTimeoutRef.current = setTimeout(() => sync(value), 500);
  }

  return (
    <input className='Message2Opponent'
      placeholder='Message to opponent'
      value={value}
      maxLength={150}
      onChange={e => update(e.target.value)}
    />
  )
}