import './GameSelector.css';
import React, { useContext } from 'react';
import { GamesContext } from '../../../context/games';

import { Color, Player } from '../../../components/Checkers';
import Loading from '../../../components/Loading';

export default function GameSelector({ yours, opponents, onClick }) {

    const gamesList = useContext(GamesContext).gamesList;
    if (gamesList === null)
        return <Loading />

    const yoursList = gamesList.filter(game => game.status === yours)
        .map(game => <Selectable game={game} key={game.uuid} onClick={onClick} />)

    const opponentsList = gamesList.filter(game => game.status === opponents)
        .map(game => <Selectable game={game} key={game.uuid} onClick={onClick} />)

    return (
        <div className="GameSelector">
            {yoursList}
            {opponentsList.length > 0 && <Separator counter={opponentsList.length} />}
            {opponentsList}
        </div>
    )
}

function Selectable({ game, onClick }) {
    const myColor = game.myColor;
    const opponentColor = Color.opposite(myColor);
    const opponentName = game.opponentName;

    return (
        <div className='Selectable' onClick={() => onClick(game.uuid)}>
            <div className='Title'>
                <Player color={myColor} />
                <i>vs</i>
                <Player color={opponentColor} name={opponentName} />
            </div>
            <q>{game.message}</q>
        </div>
    )
};

function Separator({ counter }) {
    return (
        <div className="Separator">
            waiting for opponent ({counter})
        </div>
    )
}